------------------------------------------------------------------------------------------
-- Whammy
-- Created By:  Jaycyn of Hyjal
--
-- What is it: Heal, Decurse and more with a point and click
-- License   : Distribute as you wish, please give credit to the author, please send
--           :   bugs and suggestions instead of modifying code yourself.
------------------------------------------------------------------------------------------

local WhammyVers = "0.4.1";
local WhammyDate = "05/24/2007";

W_Options={
	["Mute"]=nil, --ailment muting
	["ShowPetButtons"]=nil,
	["PetButtonLocation"]=nil,
	["FrameShape"]=nil,
	["Layout"]=nil,
	["PlayerButtonOnly"]=nil,
	["ShowSummary"]=nil,
	["ShowWhammy"]=nil, --is whammy visible
	["ButtonSize"]=nil,
	["RaidButtonCount"]=nil
}

W_kUnitButtonWidth = 78; --the standard width of each unit button
W_kUnitButtonHeight = 20;

W_kUnitButtonWidthSmall = 32;
W_kUnitButtonHeightSmall = 20;

W_kUnitDead = 0; --used when changing the frame texture to different states
--W_kUnitAlive = 1;
W_kUnitHealthAlert = 2;
W_kUnitHealthOK = 3;
W_kUnitDisconnected = 4;

W_kPetDead = 10;
--W_kPetAlive = 11;
W_kPetHealthAlert = 12;
W_kPetHealthOK = 13;
W_kPetDisconnected = 14;

W_kPlayer = 0; --used to set the whammy frame size
W_kParty = 1;
W_kRaid = 2;

W_kHealthBarHeight = 3;

W_kCurseWidth = W_kUnitButtonWidth; --width and height of the cursebar
W_kCurseHeight = 6;

W_kCurseLeftSideWidth = 2;
W_kCurseRightSideWidth = 2;
W_kCurseSidesHeight = W_kUnitButtonHeight - W_kHealthBarHeight;

W_kButtonSizeStandard = 0;
W_kButtonSizeSmall = 1;

W_kFrameShapeV = 0;
W_kFrameShapeH = 1;

W_kPetButtonBottom = 0;
W_kPetButtonRight = 1;

W_kPetButtonNotVisible = 0;
W_kPetButtonVisible = 1;

W_kButtonGutter = 3;

-- below are functions dealing with clicking in the unit buttons to cast spells
--   note that the default action for each button is assigned when the main frame loads.
function W_UnitButton_OnLoad()
	local b
	--W_ShowMessage("Unit_Button_Onload");
	this:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp"); -- the buttons need to see the mouse clicks
	
	--set the positioning for the Magic, Curse, Disease and Poison indicators for the Unit
	local y = 3;
	local thisButton = this:GetName();
	b = getglobal( thisButton.."_Curse_TextM")
	b:SetPoint( "TOPLEFT", thisButton, -1, y)
	b:SetTextColor( 1, 0, 0 );
	
	b = getglobal( thisButton.."_Curse_TextC")
	b:SetPoint( "TOPLEFT", thisButton, 10, y)
	b:SetTextColor( 1, 0, 0 );
	
	b = getglobal( thisButton.."_Curse_TextD")
	b:SetPoint( "TOPLEFT", thisButton, 17, y)
	b:SetTextColor( 1, 0, 0 );
	
	b = getglobal( thisButton.."_Curse_TextP")
	b:SetPoint( "TOPLEFT", thisButton, 26, y)
	b:SetTextColor( 1, 0, 0 );
end



--
-- Handle Unit Button Mouse actions
--
function W_UnitButton_MouseDown() --MouseDown and up just highlight the unit name when clicked. arg1 is the LeftButton, etc. unused
	local buttonName = this:GetName(); --get the name of the unit button that triggered this event
	getglobal(buttonName.."_UnitName"):SetFontObject( W_kFontNameHighlight );
end

function W_UnitButton_MouseUp() -- unhighlight the name
	local unitID = this:GetAttribute("unit"); --get the name of the unit assigned to this button
	local buttonName = this:GetName(); --get the name of the button
	local online, isDead, isAUnit;
	
	if ( unitID ) then --if this is a valid unit then
		online = UnitIsConnected( unitID ); --get the unit status assigned to this button
		isDead = W_IsDead( unitID );
		isAUnit = W_IsUnit( unitID ); -- see if this is a unit instead of a pet
		
		if ( isAUnit ) then -- this is a unit and not a pet - adjust the color of the text part of the frame
			myTextFrame = getglobal( buttonName.."_UnitName" );
			if ( online ) then
				if ( UnitIsUnit( unitID, "player") ) then --if they are online, color the player one way, and the rest of the units another
					myTextFrame:SetFontObject( W_kFontNamePlayer );
				else --someone else in the party or raid
					myTextFrame:SetFontObject( W_kFontNameOthers );
				end
			else --offline, show their text disabled
				myTextFrame:SetFontObject( W_kFontNameDisable );
			end
		end
	end
end
-- End Handle Unit Button Mouse actions



--
-- below are items dealing with the main Whammy Frame: registering for events, mouse clicks etc
--
function W_Frame_Onload(self)
	-- W_ShowMessage("Frame_Onload");

	-- Register the game events neccesary for our functionality
	self:RegisterEvent("ADDON_LOADED");
	--self:RegisterEvent("VARIABLES_LOADED");
	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("PLAYER_AURAS_CHANGED");
	self:RegisterEvent("PARTY_MEMBERS_CHANGED");
	self:RegisterEvent("RAID_ROSTER_UPDATE");
	self:RegisterEvent("UNIT_HEALTH");
	self:RegisterEvent("UNIT_MANA");
	self:RegisterEvent("PLAYER_LOGIN");
	self:RegisterEvent("PLAYER_REGEN_DISABLED"); --use this to know if we enter combat so we can disable the drop downs
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	--self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
	--					CHAT_MSG_SPELL_AURA_GONE_SELF
	
	
--self:RegisterEvent("PET_UI_UPDATE")
--self:RegisterEvent("PLAYER_PET_CHANGED")
--self:RegisterEvent("PET_BAR_CHANGED")
--self:RegisterEvent("UNIT_PET")
	
	
	-- set up the slash commands
	SlashCmdList["WHAMMY"] = W_Slash_Command_Handler;
	SLASH_WHAMMY1 = "/whammy";
	SLASH_WHAMMY2 = "/wham";
end

-- Handle stuff relevent to the Whammy Frame
function W_Frame_MouseDown(self) --this toggles the config frame
	--W_ShowMessage("Frame_MouseDown");
	
	if arg1 == "LeftButton" and IsShiftKeyDown() then -- drag the frame
		self:StartMoving();
	elseif arg1 == "RightButton" and IsShiftKeyDown() then --need to show a frame that lets the user configure the dropdowns etc
		W_Config_Frame_Toggle();
	end
end

-- Based upon our registered events, we want different things to happen when each event occurs.
function W_Frame_OnEvent(self, event, ...)
	--W_ShowMessage( event );
	
	if ( event == "ADDON_LOADED" and arg1 == "Whammy") then -- the event occurs after the saved vars are loaded
		W_ShowMessage("Version: "..WhammyVers.." "..WhammyDate.." loaded"); --" V0.21a 1/6/2007 Loaded");
		W_ShowMessage("    /Whammy to show/hide Whammy");
		this:UnregisterEvent("ADDON_LOADED");
		gWhammyIsLoaded = true;
		W_Handle_Addon_Loaded_Event(); --set up button frames, etc
	end
	
	if ( gWhammyIsLoaded == true ) then --only take action on these events if Whammy is loaded
		if ( event == "VARIABLES_LOADED" ) then --this event occurs after we load in any saved variables. Don't do anything here. functio TBD
			--W_ShowMessage("Variables Loaded Event");
		elseif ( event =="PLAYER_LOGIN") then
			if ( GetNumRaidMembers() == 0 ) then --not a raid
				W_Handle_Party_Members_Changed_Event(); --set the frame to the appropriate size for either player or party
			else -- its a raid, set up the frame
				W_Handle_Raid_Roster_Update_Event();
			end
			
			
		elseif ( event == "PLAYER_AURAS_CHANGED" ) then --watch for this player
			W_Handle_Aura_Event("player");
		elseif ( event == "PLAYER_REGEN_DISABLED" ) then --we have entered combat
			W_Handle_Combat( true );
		elseif ( event == "PLAYER_REGEN_ENABLED" ) then --we have left combat
			W_Handle_Combat( false );
			
			
		elseif ( event == "PARTY_MEMBERS_CHANGED" ) then
			if ( GetNumRaidMembers() == 0 ) then --we only care about this event if we are not in a raid since that will
				W_Handle_Party_Members_Changed_Event(); --    generate an event as well
			end
		elseif ( event == "RAID_ROSTER_UPDATE" ) then
			W_Handle_Raid_Roster_Update_Event();
			
			
		elseif ( event == "UNIT_AURA" and arg1 ~="target" and arg1 ~= "mouseover" and arg1 ~="focus" and arg1 ~="npc" and arg1 ~= "NPC" ) then --handle curses
			if ( W_IsUnit( arg1 ) ) then --if this is a unit then handle the event
				W_Handle_Aura_Event( arg1 ); --arg1 is the unitID of the unit whose buff changed
			else -- it was a pet, see if the pet buttons are showing
				if ( W_Options.ShowPetButtons ) then --and the pet buttons are visible then handle the aura event
					W_Handle_Aura_Event( arg1 ); --arg1 is the petID of the pet whose buff changed
				end
			end
		elseif ( event == "UNIT_HEALTH" and arg1 ~= "target" and arg1 ~= "mouseover" and arg1 ~="focus"  and arg1 ~= "npc" and arg1 ~= "NPC" ) then
			if ( W_IsRaid() and strfind(arg1, "raid" ) ) then --health events are called twice for a raid member that is also in your party. So, if you are
																-- in a raid, filter out the events for player and party members
					W_Handle_Health_Event( arg1 ); --arg1 is raidX, raidpetX only
			else -- not in a raid, allow player, party1-4 partypet1-4 to pass
				W_Handle_Health_Event( arg1 ); --arg1 is player, pet, partyX, partypetX, raidX, raidpetX
			end
		
		elseif ( event == "UNIT_MANA" and arg1 ~= "target" and arg1 ~= "mouseover" and arg1 ~="focus"  and arg1 ~= "npc" and arg1 ~= "NPC" ) then
			W_Handle_Unit_Mana_Event(); --update the total mana for player, party or raid, this function is in WhammySummary.lua
			-- note that we don't care about individual mana, just the mana for the entire player, party or raid
		end
	end
	
	
--	if ( event == "PET_UI_UPDATE" or event == "PLAYER_PET_CHANGED" or event == "PET_BAR_CHANGED"or event == "UNIT_PET") then
--		W_ShowMessage( event.."  "..arg1 );
--	end
end

--------
--------  EVENT HANDLERS --------
--------

-- Handle changes to the party or raid
function W_Handle_Addon_Loaded_Event()
	local frame;

	if ( W_gSavedSpells == nil ) then W_Set_Saved_Spells(); end --if no saved spells were loaded. These are items the user selected in dropdowns
	if ( W_gSavedMacros == nil ) then W_gSavedMacros = {}; end --if the user has selected a macro in a drop down, this is where it is saved to
	if ( W_gDropDownCustomList == nil ) then W_gDropDownCustomList = {}; end --stores users custom list selections for the dropdowns
	if ( W_gCustomList == nil ) then W_gCustomList = {}; end --this is where custom list items are stored
		
	W_Add_Drop_Down_Options();
	
	if ( W_gHealthLevel == nil ) then W_gHealthLevel = "Off"; end --the health level alert drop down - off by default
		
	WhammyOptionsFrame_Init(); --init all of the options the player has selected via checkboxes
	
	W_Create_Unit_Buttons(); --create all of the unit buttons and also creates all of the potential pet buttons
	W_Create_Pet_Buttons();
	
	if ( W_Options.ShowWhammy ) then --if the last time the player played, Whammy was visible, keep it visible
		frame = WhammyFrame;
		if ( frame ) then
			frame:Show();
		end
	end
end

-- called after the spell tables have either been loaded or defaulted, to add any addtional button options: Trade, Whisper, Target etc
function W_Add_Drop_Down_Options()
	local charClass = W_GetCharClass("player");
	local tableName = ("W_"..charClass.."SpellTable"); --figure out which table to use based on char type
	local t = getglobal(tableName); -- a reference to the table
	table.insert(t, W_kMenuTrade);
	table.insert(t, W_kMenuWhisper);
	table.insert(t, W_kMenuTarget);
	table.insert(t, W_kMenuRunMacro);
	table.insert(t, W_kMenuCustomList);
end


-- called when the party members change (add, leave) - will only be called for solo and party events
function W_Handle_Party_Members_Changed_Event()
	--W_ShowMessage("Party members changed event");
	local i, numberOfPartyMembers;
	local unitID, petID, buttonName;
	local numberOfPartyMembers = GetNumPartyMembers(); -- is there anyone else in the party? excluding the player ( buttons2-5)
	
	W_ResetAllButtons();
	
	W_Set_Unit_Button_Name("W_Unit_Button1","player"); --set the unit button 1 to this player

	if ( W_CharHasAPet("player") ) then
		W_Set_Unit_Button_Name( "W_Pet_Button1", "pet" ); --set the button to this pet and adds the spells
	end
					
	if ( numberOfPartyMembers == 0 ) then -- the player is by themself, reset the other buttons and frame size
		W_Set_Whammy_Frame_Size();
	else -- player is in a party, populate the other player buttons and set the frame size
		for i = 1,4 do --1 through 4 are the other party members i.e. party1 is the first other party member etc
			unitID = "party"..i;
			if UnitExists( unitID ) then --if this is a valid unit then
				buttonName = "W_Unit_Button"..(i+1); --set the button to this unit
				W_Set_Unit_Button_Name( buttonName, unitID );
								
				if ( W_CharHasAPet( unitID ) ) then
					petID = "partypet"..i
					buttonName = "W_Pet_Button"..(i+1);
					W_Set_Unit_Button_Name( buttonName, petID ); --set the button to this pet and adds the spells
				end
			end
		end
		W_Set_Whammy_Frame_Size();
	end
end

--called when the raid roster changes or upon player login if they are in a raid
function W_Handle_Raid_Roster_Update_Event()
	--W_ShowMessage("Raid Roster Update");
	local name, rank, subgroup, level, class, fileName, zone, online, isDead;
	local unitCount, pos, id;
	local unitButtonName, unitID, petButtonName, petID;
	local subGroupUnitCount = {0,0,0,0,0,0,0,0}; --keep track of how many raid members are in each raid group. Starts at 0 (5 per group)
	W_gUnitButtonTable = {}; --clear out the global lookup table that links player unitID'S to the unit buttons they are in,
	W_gPetButtonTable = {};
	
	local numRaidMembers = GetNumRaidMembers();
	
	W_ResetAllButtons();

	if ( numRaidMembers == 0 ) then -- if there are no raid members, reset size etc to just the player
		W_Set_Unit_Button_Name("W_Unit_Button1","player"); --set the unit button 1 to this player

		if ( W_CharHasAPet("player") ) then
			W_Set_Unit_Button_Name( "W_Pet_Button1", "pet" ); --set the button to this pet and adds the spells
		end
		W_Set_Whammy_Frame_Size(); --set the frame size to just player
	else
		W_Set_Whammy_Frame_Size();
		for id = 1,numRaidMembers do --walk through all the raid members
			name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(id);	--get their name and subgroup
			subGroupUnitCount[ subgroup ] = subGroupUnitCount[ subgroup ] + 1; --keep track of how many units are in each subgroup
	
			unitCount = subGroupUnitCount[ subgroup ]; --get the next unit position based on the unit count of the subgroup
			
			pos = ( ( (subgroup-1) * 5) + unitCount ) -- first subgroup, first position will be 1, 8th subgroup, first position will be 36
			
			unitButtonName = "W_Unit_Button"..pos
			unitID = "raid"..id
			W_Set_Unit_Button_Name( unitButtonName, unitID );
			W_gUnitButtonTable[ unitID ] = unitButtonName; --add the buttons that are occupied to the table so we can look them up later
			if ( W_CharHasAPet( unitID ) ) then
				petButtonName = "W_Pet_Button"..pos;
				petID = "raidpet"..id
				W_Set_Unit_Button_Name( petButtonName, petID ); --set the button to this petID and adds the spells
				W_gPetButtonTable[ petID ] = petButtonName; -- store this in the pet button table so raidpet1 = W_Pet_Button1 etc
			end
		end
	end
end

-- handle unit health events for player or someone in the players party (1-4) or raid (1-40)
-- if in a raid, this will be called twice for player and party members. once as a player, again as a raid member and
--    for party members, once for the party and again for the raid
function W_Handle_Health_Event( unitID )
	--W_ShowMessage("Handle Unit Health Event for "..unitID);
	local myButtonName, partyMemberNum;
	local online, isDead;
	local myTextFrame;
	local isAUnit = W_IsUnit( unitID ); -- is this a unit as opposed to a pet
	local healthTargetPercent;
	local buttonWidth, maxHealth, currentHealth, healthFraction, healthBarAmount, healthTargetPercent;
	
	myButtonName = W_Get_Button_Name_From_UnitID( unitID ); --returns W_Unit_ButtonX or W_Pet_ButtonX
	
	if ( myButtonName ~= nil ) then
		online = UnitIsConnected( unitID );
		isDead = W_IsDead( unitID );
		
		maxHealth = UnitHealthMax(unitID);
		currentHealth = UnitHealth(unitID);
		healthFraction = currentHealth / maxHealth;
		
		buttonWidth = getglobal( myButtonName ):GetWidth(); --get the width of the button so we can calculate the healthbar width

		if ( healthFraction > 1 ) then --needed to add this becuase there was a bug in reading the pet health. Fixed in 2.0.10 3/20/07
			healthFraction = 1;
		end
		if ( isAUnit ) then -- this is a unit and not a pet - adjust the color of the text part of the frame
			myTextFrame = getglobal( myButtonName.."_UnitName" );
			if ( online ) then
				if ( UnitIsUnit( unitID, "player") ) then --if they are online, color the player one way, and the rest of the units another
					myTextFrame:SetFontObject( W_kFontNamePlayer );
				else
					myTextFrame:SetFontObject( W_kFontNameOthers );
				end
			else --offline, show their text disabled
				myTextFrame:SetFontObject( W_kFontNameDisable );
			end
			
			if ( isDead ) then --dead so show no health and adjust button texture
				W_Set_Unit_Button_Texture( myButtonName, W_kUnitDead ); --unit is dead, grey out the unit button texture
				W_Set_Unit_Health_Bar_Value( myButtonName, 1); --set the healthbar to 1 - 0 doesn't work
			else -- unit is alive
				W_Set_Unit_Health_Bar_Value( myButtonName, healthFraction ); --update the health bar
				
				if W_gHealthLevel ~= "Off" then --if the user wants to be notified for low health, let them know if there is low health
					healthTargetPercent = tonumber(W_gHealthLevel)/100; --health level is the % health selected in the drop down
					if ( healthFraction < healthTargetPercent ) then -- is the health below they threshold?
						W_Set_Unit_Button_Texture( myButtonName, W_kUnitHealthAlert ) --turn the unit button red
					else --good health, set to default color
						W_Set_Unit_Button_Texture( myButtonName, W_kUnitHealthOK )
					end
				else
					W_Set_Unit_Button_Texture( myButtonName, W_kUnitHealthOK);
				end
			end
			

		else --it's a pet health event
			if ( isDead ) then --dead so show no health and adjust button texture
				W_Set_Unit_Button_Texture( myButtonName, W_kPetDead ); --pet is dead, grey out the button texture
				W_Set_Unit_Health_Bar_Value( myButtonName, 1);
			else -- pet is alive
				W_Set_Unit_Health_Bar_Value( myButtonName, healthFraction );
				
				if W_gHealthLevel ~= "Off" then
					healthTargetPercent = tonumber(W_gHealthLevel)/100;
					if ( healthFraction < healthTargetPercent ) then
						W_Set_Unit_Button_Texture( myButtonName, W_kPetHealthAlert )
					else
						W_Set_Unit_Button_Texture( myButtonName, W_kPetHealthOK )
					end
				else
					W_Set_Unit_Button_Texture( myButtonName, W_kPetHealthOK);
				end
			end
			
		end
		
	end
end

function W_IsDead( unitID )
	local isDead = (   UnitIsDeadOrGhost( unitID ) or ( UnitHealth( unitID ) <= 0 ) or UnitIsCorpse( unitID ) or UnitIsDead( unitID )   )
	
	return isDead
end


function W_Set_Unit_Button_Texture( buttonName, whichTexture ) --set the texture of the entire unit button, either W_Unit_Button1 or W_Pet_Button1
	--W_ShowMessage("Set Unit Button Texture for button "..buttonName.." texture is "..whichTexture );
	local r, g, b, a;
	local buttonTextureName
	
	if ( buttonName ~= nil) then
		if ( whichTexture == W_kUnitHealthOK ) then
			r = 1; g = 1; b = 1; a = .2;
			
		--elseif ( whichTexture == W_kUnitAlive ) then
		--	r = 1; g = 1; b = 1; a = .2;	
			
		elseif ( whichTexture == W_kUnitDead ) then
			r = .1; g = .1; b = .1; a = .8;
			
		elseif ( whichTexture == W_kUnitHealthAlert ) then
			r = 1; g = 0; b = 0; a = .5;
			
		elseif ( whichTexture == W_kUnitHealthOK ) then
			r = 1; g = 1; b = 1; a = .2;

		elseif ( whichTexture == W_kUnitDisconnected ) then
			r = 0; g = 0; b = 0; a = 0;	
			
		--elseif ( whichTexture == W_kPetAlive ) then
			--r = .2; g = .2; b = .2; a = 1;	

		elseif ( whichTexture == W_kPetDead ) then
			r = .1; g = .1; b = .1; a = 1;
			
		elseif ( whichTexture == W_kPetHealthAlert ) then
			r = 1; g = 0; b = 0; a = 1;
			
		elseif ( whichTexture == W_kPetHealthOK ) then
			r = 1; g = 1; b = 1; a = .2;
			--r = 0; g = 0; b = 1; a = 1;
		elseif ( whichTexture == W_kPetDisconnected ) then
			r = 0; g = 0; b = 0; a = 0;	
			
		else
			r = 1; g = 1; b = 1; a = .2;
		end
		
		buttonTextureName = buttonName.."_Texture";
		--W_ShowMessage("       buttonTextureName="..buttonTextureName)
		getglobal( buttonTextureName ):SetTexture( r, g, b, a );
	end
end

function W_Set_Unit_Health_Bar_Value( buttonName, healthFraction ) --set the level of the health bar on the unit button
	local buttonWidth = getglobal( buttonName ):GetWidth() --the actual width of the button
	local buttonHealthBarName = ( buttonName.."_HealthBar" ); --so we have W_Unit_Button1_HealthBar
	local newWidth = buttonWidth * healthFraction;
	getglobal( buttonHealthBarName ):SetWidth( newWidth ); --adjust the width of the healthbar
	
	W_Update_Total_Health(); -- update the total health bar for the player, party or raid
end

function W_Set_Unit_Button_Curse_Texture( buttonName, isDebuffed, debuffType) --set the texture of the curse area on the unit button
	-- type will be Magic, Disease, Poison or Curse or nil is none
	local bTop = getglobal( buttonName.."_Curse_Texture" );
	local bLeft = getglobal( buttonName.."_CurseLeft_Texture")
	local bRight = getglobal( buttonName.."_CurseRight_Texture" );
	local bMagicText = getglobal( buttonName.."_Curse_TextM" );
	local bCurseText = getglobal( buttonName.."_Curse_TextC" );
	local bDiseaseText = getglobal( buttonName.."_Curse_TextD" );
	local bPoisonText = getglobal( buttonName.."_Curse_TextP" );

	if ( isDebuffed ) then --there is a debuff, so set the texture. Make sure to maintain
		bTop:SetTexture( .97, 1, .38, 1 ); --yellow
		bLeft:SetTexture( .97, 1, .38, 1 ); --yellow
		bRight:SetTexture( .97, 1, .38, 1 ); --yellow
		
		if (debuffType == "Magic") then --if it was magic then set the magic text otherwise clear it
			bMagicText:SetText("m");
		end
		
		if (debuffType == "Curse") then
			bCurseText:SetText("c");
		end	
			
		if (debuffType == "Disease") then
			bDiseaseText:SetText("d");
		end
			
		if (debuffType == "Poison") then
			bPoisonText:SetText("p");
		end

	else --no buff, remove the texture and set the unit name back
		bTop:SetTexture( nil );
		bLeft:SetTexture( nil );
		bRight:SetTexture( nil );
		
		bMagicText:SetText(nil); --if there are no debuffs, clear them all
		bCurseText:SetText(nil);
		bDiseaseText:SetText(nil);
		bPoisonText:SetText(nil);
	end
end

-- called when the player/pet, a party/partypet or a raid member/raid pet  buffs/debuffs change.
function W_Handle_Aura_Event( unitID )
	--W_ShowMessage( "Unit Aura Event for unitid "..unitID);
	local debuffName, debuffRank, debuffTexture, debuffApplications, debuffType;
	local debuffNum = 1;
	local showDispellable = 1; -- 1 = only return debuffs that are dispellable, 0 = all buffs
	local foundOne, unitButtonName;

	unitButtonName = W_Get_Button_Name_From_UnitID( unitID ); --returns W_Unit_ButtonX or W_Pet_ButtonX
	
	if ( unitButtonName ~= nil) then
		W_Set_Unit_Button_Curse_Texture( unitButtonName, false, nil); --start by clearing the curse bar and letters
		while ( UnitDebuff( unitID, debuffNum) ) do -- while the unit has debuffs, walk through them as see if there are any we can fix
			debuffName, debuffRank, debuffTexture, debuffApplications, debuffType = UnitDebuff( unitID, debuffNum, showDispellable );
			if (debuffTexture ~= nil) then --for each one found, pass the type which will then display the corresponding letter M, D, P or C
				W_Set_Unit_Button_Curse_Texture( unitButtonName, true, debuffType); --type will be Magic, Disease, Poison or Curse
				if ( W_Options.Mute == false ) then -- if we are not muted, play a sound
					PlaySoundFile("interface\\addons\\Whammy\\AilmentSound.mp3");
				end
			end
			debuffNum = debuffNum + 1;
		end
	end
end

--called when health or aura events occur in a raid
--   determine which button is associated with the passed in unitID. Pass in player, pet, party1-4, partypet 1-4, raid 1-40 or raidpet1-40
--   return the button name that refers to this unit or pet either W_Unit_ButtonX or W_Pet_ButtonX
function W_Get_Button_Name_From_UnitID( unitID )
	local b;
	local id;
	
	--W_ShowMessage( "Getting Button name for unit "..unitID );
	
	if ( unitID == "player" ) then
		b = "W_Unit_Button1"
	elseif ( unitID == "pet" ) then
		b = "W_Pet_Button1"
	else --we are either in a party or a raid, and could be a unit or a pet
		if ( W_IsRaid() ) then
			if ( strfind(arg1, "raid" ) ) then --does the unitID contain the text raid, then let it pass
				b = W_Get_Raid_Unit_Button_From_UnitID( unitID ); --look up which button this raid unit is in via the global table
			end
		else --its a party
			if ( W_IsUnit( unitID ) ) then --its a player unit
				id, numberFound = gsub( unitID, "party([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
				b = "W_Unit_Button"..(id+1);
			else --its a pet
				id, numberFound = gsub( unitID, "partypet([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
				b = "W_Pet_Button"..(id+1)
			end
		end
	end

	return b
end

-- called when we enter or leave combat. Enable or disable the drop downs depending on combat status
function W_Handle_Combat( isInCombat )
	local dropDownName;
	
	if ( isInCombat ) then
		for i = 1,W_kMaxDropDownButtons do --walk through the drop downs and enable or disable them depending on if we are in combat
			dropDownName = "W_DropDown"..i
			UIDropDownMenu_DisableDropDown( getglobal( dropDownName ) );
		end
	else
		for i = 1,W_kMaxDropDownButtons do --walk through the drop downs and enable or disable them depending on if we are in combat
			dropDownName = "W_DropDown"..i
			UIDropDownMenu_EnableDropDown( getglobal( dropDownName ) );
		end
	end
end
--
--------  END EVENT HANDLERS --------
--


--
-- functions called to set up the unit buttons to respond to mouse clicks
--
-- called from the handle raid roster update and party change events to assign unit names to unit buttons
function W_Set_Unit_Button_Name( buttonName, unitID)
	local newStringWidth;
	local b, unitName;
	--W_ShowMessage( "Setting Unit Button "..buttonName.." to "..unitID);
	
	getglobal(buttonName):SetAttribute("unit", unitID); --set the assignment of the button to the passed in unit  (need to do this first)
	
	if ( W_IsUnit( unitID ) ) then --if it is a unit instead of a pet, put the units name in the button
		b = getglobal( buttonName.."_UnitName" );
		unitName = UnitName(unitID); -- unit ID will be "player" or "party1" or "raid1", and this will give us the actual name
		b:SetText(unitName); --set the text of the passed in button to the unit name
		
		--adjust the width of the string so it fits in the unit frame
		W_Adjust_Unit_Name_Width( buttonName );
		
		if ( UnitIsUnit( unitID, "player" ) ) then -- if this button is the player button, make it stand out so it is more easily picked out
			b:SetFontObject( W_kFontNamePlayer );
		else
			if ( UnitIsConnected( unitID ) ) then --if the unit is connected, standard font color
				b:SetFontObject( W_kFontNameOthers );
			else --if the unit disconnects then change the font color to disconnected color
				b:SetFontObject( W_kFontNameDisable );
			end
		end
	else --it was a pet, make sure the button is visible
		getglobal(buttonName):Show()
	end
	
	W_Set_Unit_Button_Click( buttonName );
end

-- adjust the unit name to fit properly in the button
function W_Adjust_Unit_Name_Width( buttonName )
	local buttonWidth, stringWidth, newStringWidth, b;

	buttonWidth = getglobal(buttonName):GetWidth(); --the width of the button
	
	b = getglobal( buttonName.."_UnitName" ); -- a reference to the unit name text of the button
	stringWidth = b:GetStringWidth(); --the width of the unit name

	if ( stringWidth > buttonWidth ) then --if the unit name is wider than the button, then truncate the unit name
		newStringWidth = buttonWidth * .90 -- the new string width will be 80% of the button width
		b:SetWidth( newStringWidth );
	else --the unit name fit within the box so reset the string width value
		b:SetWidth( buttonWidth );
	end
end

--walk through and update all of the unit buttons to reflect a change in spell or unit etc
function W_Update_Unit_Buttons() 
	for i = 1,W_kMaxUnitButtons do --now that we changed a spell, walk through and update all of the unit buttons to reflect the change
		W_Set_Unit_Button_Click( "W_Unit_Button"..i );
		W_Set_Unit_Button_Click( "W_Pet_Button"..i);
	end
end

--called from handle raid roster update, dropdown selection change events and default button click.
--      Assign mouse clicks to the passed in unit button
function W_Set_Unit_Button_Click( buttonName )
	--W_ShowMessage("setting ".. unitButtonName);
	W_Set_Click("", "1", buttonName, "W_DropDown1" ); --Left Click is DropDown1
	W_Set_Click("", "2", buttonName, "W_DropDown3" ); -- Right Click is DropDown3
	W_Set_Click("", "3", buttonName, "W_DropDown2" ); --Middle Button Click is DropDown2
	W_Set_Click("ctrl", "1", buttonName, "W_DropDown4" ); --Ctrl Left
	W_Set_Click("ctrl", "2", buttonName, "W_DropDown6" ); --Ctrl Right
	W_Set_Click("ctrl", "3", buttonName, "W_DropDown5" ); --Ctrl Middle
	W_Set_Click("shift", "1", buttonName, "W_DropDown7" ); --Shift Left
	W_Set_Click("shift", "2", buttonName, "W_DropDown9" ); --Shift Right
	W_Set_Click("shift", "3", buttonName, "W_DropDown8" ); --Shift Middle
	W_Set_Click("alt", "1", buttonName, "W_DropDown10" ); --Alt Left
	W_Set_Click("alt", "2", buttonName, "W_DropDown12" ); --Alt Right
	W_Set_Click("alt", "3", buttonName, "W_DropDown11" ); --Alt Middle
end

-- Called from W_Set_Unit_Button_Click to set what the mouse click does in reponse to the players frameButton
function W_Set_Click(keyPress, mouseButton, buttonName, ddMenuName )
	--keyPress will be filled if there is a keyboard button pressed along with the mouse click
	--mouseButton is: 1=Left, 2=Right, 3=Middle, 4 and 5
	--buttonName is the frame button on the screen we are assigning the clicks to
	--ddMenuName is the dropdown menu name this spell is saved for
	
	local myUnitID, myUnitName, myButton, menuItem, itemName;
	local chatType, channel;
	local thingTable, thingName, thingType;
	
	myButton = getglobal( buttonName ); --get a pointer to the unitButton that we are configuring
	
	if ( myButton:GetAttribute("unit") ~= nil ) then -- the unit attrib would be nil if it is does not have a unit assigned to it
		if ( keyPress == "ctrl" or keyPress == "shift" or keyPress == "alt") then --if there is a keypress modifier then we need to add 
			keyPress = ( keyPress.."-" ); --                                       the dash so the attribute will be properly formatted
		end
		--get the item from the saved spell table: spell, Whisper, Trade, W_kMenuRunMacro, W_kMenuCustomList
		menuItem, itemName = W_Get_Saved_MenuItem( ddMenuName );
		
		if ( menuItem == W_kMenuTrade ) then --this button will be set to trade
			myUnitID = myButton:GetAttribute("unit");
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "W_Perform_Trade");
			myButton.W_Perform_Trade = function() InitiateTrade(myUnitID); W_ShowMessage(W_kMsgOpeningTradeWindow); end;
			
		elseif ( menuItem == W_kMenuWhisper ) then
			myUnitID = myButton:GetAttribute("unit");
			myUnitName = UnitName( myUnitID );
			chatType = "WHISPER";
			channel = GetChannelName( chatType );
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "W_Perform_Chat_Message");
			myButton.W_Perform_Chat_Message = function() ChatFrame_SendTell(myUnitName); W_ShowMessage(W_kMsgWhisper); end;
			
		elseif ( menuItem == W_kMenuTarget ) then --just target the unit
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "target");
			
		elseif ( menuItem == W_kMenuRunMacro) then
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "macro");
			myButton:SetAttribute( (keyPress.."macro"..mouseButton), itemName); --which spell is actually a macro here.
			
		elseif ( menuItem == W_kMenuCustomList ) then --it's an item from the custom list
			if ( itemName ) then --make sure the item name is valid before proceeding
				thingTable = itemName; --in this case the itemName will be a table of itemName and type
				thingName = thingTable[1];
				thingType = thingTable[2];
	
				myButton:SetAttribute( (keyPress.."type"..mouseButton), thingType); --thingType is item, spell or macro
				myButton:SetAttribute( (keyPress..thingType..mouseButton), thingName)
			end

		else --we are setting the button to a spell - either one from the list or the custom list
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "spell");
			myButton:SetAttribute( (keyPress.."spell"..mouseButton), menuItem);
		end
	end
end

function W_Open_Trade_Macro()
	InitiateTrade("target");
end

function W_ResetAllButtons()
	local unitButtonName, petButtonName;
	local ubTextFrame;
	local ub, pb;


	for i = 1,W_kMaxUnitButtons do --clear out the raid buttons
		unitButtonName = "W_Unit_Button"..i
		petButtonName = "W_Pet_Button"..i
		ubTextFrame = getglobal( unitButtonName.."_UnitName" );
		ub = getglobal( unitButtonName );
		pb = getglobal( petButtonName);
		
		-- set the text of the button to -
		ubTextFrame:SetText("-");
		ubTextFrame:SetFontObject( W_kFontNameOthers );
		
		-- set the action of the button to nil
		ub:SetAttribute("type", "stop"); --set the function of the button to nothing
		ub:SetAttribute("unit", nil); --set the unit attrib (the unit name) to nothing
		
		-- set the color of the button to standard
		W_Set_Unit_Button_Texture( unitButtonName, W_kUnitHealthOK );
		
		-- set the curse of the button to off
		W_Set_Unit_Button_Curse_Texture( unitButtonName, false, nil);
		
		-- set the health bar to full health
		W_Set_Unit_Health_Bar_Value( unitButtonName, 1);
		
		
		pb:SetAttribute("type", "stop"); --set the function of the button to nothing
		pb:SetAttribute("unit", nil); --set the unit attrib (the unit name) to nothing
		pb:Hide();
		
		-- set the color of the button to standard
		W_Set_Unit_Button_Texture( petButtonName, W_kPetHealthOK );
		
		-- set the curse of the button to off
		W_Set_Unit_Button_Curse_Texture( petButtonName, false, nil);
		
		-- set the health bar to full health
		W_Set_Unit_Health_Bar_Value( petButtonName, 1);
	end
end

--
-- W_DropDownMenu stuff: the onload function defines which function init's the button (which is automatically called)
--
-- load up the action buttons with default spells if there were no saved spells
function W_Set_Saved_Spells()
	local dropDownButtonName;
	
	W_ShowMessage(W_kMsgSettingSpellsToDefault);
	if ( W_gSavedSpells == nil ) then
		W_gSavedSpells = {}; --create the table
	end
	
	local charClass = W_GetCharClass("player"); --what class of player is this: used to populate Hunter and Warlock pet buttons
	local defaultTableName = "W_"..charClass.."DefaultSpellTable"; -- the default spell table for this class ie W_PriestDefaultSpellTable

	for i = 1,W_kMaxDropDownButtons do --for each drop down button, get what it's default should be from the default tables for this char type
		dropDownButtonName = "W_DropDown"..i;
		W_gSavedSpells[ dropDownButtonName ] = getglobal(defaultTableName)[i]; -- this is W_DropDown1="Greater Heal" and W_DropDown2="Renew" etc
	end
end

--	use for troubleshooting
function W_Show_Spells() -- this function is for troubleshooting
	local value;
	
	for k, v in pairs(  W_gSavedSpells ) do --iterate over the key, values pairs from the spell table appropriate for this char
		value = W_gSavedSpells[k];
	end
end


--
-- MISC functions from here down
--

-- Display a message in the chat frame
function W_ShowMessage(aMessage)
	local r,g,b,a
	r = 0;
	g = 1;
	b = 0;
	a = .5;
	
	if ( aMessage ~= nil ) then
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: " ..aMessage, r, g, b, a);
	else
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: nil message");
	end
end



function W_Options_Button_Click()
	WhammyConfigFrame:Hide();
	WhammyOptionsFrame:Show();
end

function W_Test_Button_Click()
	--W_ShowMessage("test button");

	WhammyOptionsFrame:Show();
end

function W_Test_Onload()
	this:RegisterForClicks("LeftButtonUp"); -- the buttons need to see the mouse clicks
end

function W_Test_OnEvent()
end

function W_Test_MouseDown()
end

function W_Help_Test_Button_Click()
	--W_ShowMessage("help test button");
end

function W_Info_Button_Click() -- show/hide the info frame when the user clicks the ? button
	local frame = WhammyInfoFrame;
	if ( frame ) then
		if ( frame:IsVisible() ) then
			frame:Hide();
		else
			frame:Show();
		end
	end
end

--
-- When the info window loads, set the text that should appear
--
function W_Info_Onload(self)
	--W_ShowMessage("info onload");
	local infoText
	local kHelpFrameWidth = 400;
	
	WhammyInfoFrame:SetWidth( kHelpFrameWidth ); --adjust the size of the frame
	WhammyInfoFrame_Heading:SetText("Whammy "..WhammyVers);
	
	-- note that the height of the info frame will adjust around the height of the text. Did this for localization purposes in case another language takes
	--    more or less space to say the same thing
	local t = WhammyInfoFrame_Help_Text;
	
	t:SetPoint("TOPLEFT", "WhammyInfoFrame", 10, -35); --set the starting position of the help text
	t:SetWidth( kHelpFrameWidth - 8); --set the width of the text
	t:SetText( W_kHelpText ); --set the text from the static text
	local textHeight = t:GetHeight(); --get the height of the text so we can adjust the frame height around it
	WhammyInfoFrame:SetHeight( textHeight + 40 ) --set the frame height to the height of the text + space for the heading area
end


-- called to adjust the whammy frame size depending on single player, party or raid size, and the number of buttons to be displayed
function W_Set_Whammy_Frame_Size( groupType )
	--W_ShowMessage("SetWhammyFrameSize")
	local newWidth, newHeight, numToShow;
	local marginSides = 10;
	local marginTopBottomPlayer = 24;
	local marginTopBottomParty = 24;
	local marginTopBottomRaidV = 24;
	local marginTopBottomRaidH = 24;
	local buttonName, buttonWidth, buttonHeight, petButtonWidth, petButtonHeight;
	local aGroupType;
	
	if ( groupType == nil ) then --if we didn't pass anything in, then figure out what frame size we are supposed to be
		if ( W_IsRaid() ) then --is this a raid, set up the raid frame
			aGroupType = W_kRaid;
		elseif ( W_IsParty() ) then --is this a party, set up the party frame
			aGroupType = W_kParty;
		else --its just a player, set up the player frame
			aGroupType = W_kPlayer;
		end
	else --we passed a type in instead, so use it
		aGroupType = groupType;
	end
	
	--aGroupType = W_kRaid; --this is for debugging
	
	if ( W_Options.ButtonSize == W_kButtonSizeSmall) then --buttons are small size, so set the frame small
		buttonWidth = W_kUnitButtonWidthSmall;
		buttonHeight = W_kUnitButtonHeightSmall;
	elseif ( aGroupType == W_kRaid and W_Options.RaidButtonCount == 40 and W_Options.FrameShape == W_kFrameShapeH ) then
	    --if its a 40 man raid and horizontal, force small buttons to be used and pet buttons at the bottom
		buttonWidth = W_kUnitButtonWidthSmall;
		buttonHeight = W_kUnitButtonHeightSmall;
		W_Options.ButtonSize = W_kButtonSizeSmall;
		W_Options.PetButtonLocation = W_kPetButtonBottom; --also put the pet buttons on the bottom
	else --buttons are standard size
		buttonWidth = W_kUnitButtonWidth;
		buttonHeight = W_kUnitButtonHeight;
	end
	
	if ( W_Options.ShowPetButtons ) then --if the pet buttons are visible, add width or height to the button height for calculating frame size
		petButtonWidth = W_kPetButtonWidth;
		petButtonHeight = W_kPetButtonHeight;
		
		if ( W_Options.PetButtonLocation == W_kPetButtonRight ) then --on the right so add the width of the pet button to the right of the unit button
			buttonWidth = buttonWidth + petButtonWidth;
		elseif ( W_Options.PetButtonLocation == W_kPetButtonBottom ) then --at the bottom so add the pet button height to the unit button height
			buttonHeight = buttonHeight + petButtonHeight;
		end
	else
		petButtonWidth = 0;
		petButtonHeight = 0;
	end
	
	W_Set_Button_Size(); --adjust the buttons to their new size so we can then adjust the frame around them. This is needed in case we call
						--     this method from the small/reg button change checkbox in the options frame
	
	if ( aGroupType == W_kPlayer or W_Options.PlayerButtonOnly == true ) then --single player or the player only wants to see their own button
		if ( W_Options.FrameShape == W_kFrameShapeV ) then
			newWidth = ( buttonWidth + marginSides);
			newHeight = ( buttonHeight + marginTopBottomPlayer );
		else -- frame is laid out horizontally
			newWidth = ( buttonWidth + marginSides);
			newHeight = ( buttonHeight + marginTopBottomPlayer );
		end
		numToShow = 1; --1 is default size for single player
	elseif ( aGroupType == W_kParty ) then
		if ( W_Options.FrameShape == W_kFrameShapeV ) then --frame is vertical
			newWidth = buttonWidth + marginSides;
			newHeight = ( ( buttonHeight + W_kButtonGutter ) * 5 ) + marginTopBottomParty;
		elseif ( W_Options.FrameShape == W_kFrameShapeH ) then --frame is horizontal
			newWidth = ( (buttonWidth + W_kButtonGutter) * 5 ) + marginSides;
			newHeight = buttonHeight + marginTopBottomParty;
		end
		numToShow = 5;
	elseif ( aGroupType == W_kRaid ) then --in a raid
		if ( W_Options.FrameShape == W_kFrameShapeV  ) then --Frame is vertical - more of a square in the case of a raid
			if ( W_Options.RaidButtonCount == 40 ) then
				newWidth = ( 4 * (buttonWidth + W_kButtonGutter) ) + marginSides; -- 4 columns wide
				newHeight = ( 10 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidV; --10 rows tall
			elseif ( W_Options.RaidButtonCount == 25 ) then
				newWidth = ( 3 * (buttonWidth + W_kButtonGutter) ) + marginSides; -- 2 columns wide
				newHeight = ( 10 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidV; --13 rows tall
			elseif ( W_Options.RaidButtonCount == 10 ) then
				newWidth = ( 1 * (buttonWidth + W_kButtonGutter) ) + marginSides; -- 1 column wide
				newHeight = ( 10 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidV; --10 rows tall
			elseif ( W_Options.RaidButtonCount == 5 ) then
				newWidth = ( 1 * (buttonWidth + W_kButtonGutter) ) + marginSides; -- 1 column wide
				newHeight = ( 5 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidV; --10 rows tall
			elseif ( W_Options.RaidButtonCount == 1 ) then
				newWidth = ( 1 * (buttonWidth + W_kButtonGutter) ) + marginSides; -- 1 column wide
				newHeight = ( 1 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidV; --1 row tall
			else --this should never happen, but in case it does, default to full size
				newWidth = ( 4 * (buttonWidth + W_kButtonGutter) ) + marginSides; -- 4 columns wide
				newHeight = ( 10 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidV; --10 rows tall
				--W_ShowMessage("not supposed to see this");			
			end
		elseif ( W_Options.FrameShape == W_kFrameShapeH ) then --Frame is horizontal - buttons are small so they will fit horizontally
			if ( W_Options.RaidButtonCount == 40 ) then
				newWidth = ( (buttonWidth + W_kButtonGutter) * 20) + marginSides; --20 buttons wide. Remember they are forced as small buttons, pets on bottom
				newHeight = ( 2 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidH; --two buttons tall
			elseif ( W_Options.RaidButtonCount == 25 ) then
				newWidth = ( (buttonWidth + W_kButtonGutter) * 13) + marginSides; --13 buttons wide, bottom row has 12 buttons (13+12=25)
				newHeight = ( 2 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidH; --two buttons tall
			elseif ( W_Options.RaidButtonCount == 10 ) then
				newWidth = ( (buttonWidth + W_kButtonGutter) * 10) + marginSides; --10 buttons wide
				newHeight = ( 1 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidH; --one button tall
			elseif ( W_Options.RaidButtonCount == 5 ) then
				newWidth = ( (buttonWidth + W_kButtonGutter) * 5) + marginSides; --5 buttons wide
				newHeight = ( 1 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidH; --one button tall
			elseif ( W_Options.RaidButtonCount == 1 ) then
				newWidth = ( (buttonWidth + W_kButtonGutter) * 1) + marginSides; --1 buttons wide
				newHeight = ( 1 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidH; --one button tall
			else --this should never happen, but in case it does, default to full size
				newWidth = ( (buttonWidth + W_kButtonGutter) * 20) + marginSides; --20 buttons wide
				newHeight = ( 2 * ( buttonHeight + W_kButtonGutter) ) + marginTopBottomRaidH; --two buttons tall
				--W_ShowMessage("not supposed to see this");			
			end
		end
		numToShow = W_Options.RaidButtonCount;
	end
	
	for i = 1,numToShow do --show the unit buttons that need to be seen
		buttonName = "W_Unit_Button"..i
		getglobal( buttonName ):Show();
	end
	
	for i = numToShow+1,40 do --hide the unused unit buttons
		buttonName = "W_Unit_Button"..i
		getglobal( buttonName ):Hide();
		buttonName = "W_Pet_Button"..i
		getglobal( buttonName):Hide()
	end
	
	WhammyFrame:SetWidth( newWidth ); --adjust the size of the frame
	WhammyFrame:SetHeight( newHeight );
	
	W_Set_Button_Position( aGroupType ); --layout the buttons correctly to fit in the curent frame size
	
	--adjust the width of the Whammy title so it fits.
	WhammyFrame_Title:SetText( "Whammy" ); --set it to Whammy first so we can compare it and make sure it fits in the space
	local whammyTitleWidth = WhammyFrame_Title:GetStringWidth();
	local whammyFrameWidth = WhammyFrame:GetWidth();
	local borderSize = 5; --the size of the border around the frame, and then add a bit
	if ( whammyTitleWidth > (whammyFrameWidth-borderSize) ) then --if the Whammy title is wider than will fit in the frame, truncate to just a W
		WhammyFrame_Title:SetText( "W" );
	else --otherwise, set it to Whammy
		WhammyFrame_Title:SetText( "Whammy" );
	end
	
end

function W_Set_Button_Position( aGroupType )
	--W_ShowMessage("Setting Whammy Button Position")
	local buttonWidth, buttonHeight, buttonNum, numCols, numRows, numHalves, half;
	local leftMargin = 5;
	local topMargin = 18;
	local buttonName, f, x, y;
	
	buttonWidth, buttonHeight = W_Get_Button_Size(); --get the actual size of each unit button, standard or small
	
	if ( W_Options.ShowPetButtons ) then
		if ( W_Options.PetButtonLocation == W_kPetButtonRight ) then
			buttonWidth = buttonWidth + W_kPetButtonWidth;
		elseif ( W_Options.PetButtonLocation == W_kPetButtonBottom ) then --on bottom by default
			buttonHeight = buttonHeight + W_kPetButtonHeight;
		end
	end
	
	if ( W_Options.FrameShape == W_kFrameShapeV ) then
		
		if ( W_Options.RaidButtonCount == 40 ) then
			numCols = 4; --4 cols across each half
			numRows = 5; --5 buttons tall
			numHalves = 2; --2 halfs, a top for groups 1-4 and a bottom for groups 5-8
		elseif ( W_Options.RaidButtonCount == 25 ) then
			numCols = 3; --3 cols across each half
			numRows = 5; --5 buttons tall
			numHalves = 2;
		elseif ( W_Options.RaidButtonCount == 10 ) then
			numCols = 1; --1 col across each half
			numRows = 5; --5 buttons tall
			numHalves = 2;
		elseif ( W_Options.RaidButtonCount == 5 ) then
			numCols = 1; --1 col across
			numRows = 5; --5 buttons tall
			numHalves = 1;		
		elseif ( W_Options.RaidButtonCount == 1 ) then
			numCols = 1; --1 col across each half
			numRows = 1; --1 buttons tall
			numHalves = 1;
		else --this should never happen
			numCols = 4; --4 cols across each half
			numRows = 5; --5 buttons tall
			numHalves = 2; --2 halfs, a top for groups 1-4 and a bottom for groups 5-8
		end
	
		buttonNum = 1;
		for half = 1,numHalves do --the standard whammy frame has a top half, groups 1-4, and a bottom half, groups 5-8
			for col = 1,numCols do --4 cols wide
				for row = 1,numRows do --5 buttons tall
					buttonName = "W_Unit_Button"..buttonNum;
					f = getglobal( buttonName );
					x = leftMargin + ( (col-1) * (buttonWidth + W_kButtonGutter) );
					y = topMargin + ( (row-1) * (buttonHeight + W_kButtonGutter) );
					f:ClearAllPoints();
					f:SetPoint("TOPLEFT", "WhammyFrame", "TOPLEFT", x, -y);
					
					petButtonName = "W_Pet_Button"..buttonNum; --adjust the pet button position
					f = getglobal( petButtonName );
					f:ClearAllPoints();
					if ( W_Options.PetButtonLocation == W_kPetButtonRight ) then
						f:SetPoint("TOPLEFT", buttonName, "TOPRIGHT", 0, 0)
					elseif ( W_Options.PetButtonLocation == W_kPetButtonBottom ) then
						f:SetPoint("TOPRIGHT", buttonName, "BOTTOMRIGHT", 0, 0)
					end
					
					buttonNum = buttonNum + 1;
				end
			end
		
		topMargin = topMargin + ( (buttonHeight + W_kButtonGutter) * 5 ); --the second set of buttons for a raid 5-8
		end
	else --it's a horizontal frame shape, so lay the buttons out side by side, left to right, in two rows - if raid we will use small buttons
		y = topMargin
		buttonNum = 1
		
		if ( W_Options.RaidButtonCount == 25 ) then --if we are doing a 25 man, we need 13 buttons on the top row and 12 on the second row
			numCols = 13; --we will have a top row of 13 buttons and a lower row of 12
		else --any other button count we will set the buttons to be two rows of 10, however, the unused buttons will be hidden so it looks right
			numCols = 20;
		end
		
		for half = 1,2 do --raids would have two rows of 20
			for col = 1,numCols do
				buttonName = "W_Unit_Button"..buttonNum;
				f = getglobal( buttonName );
				x = leftMargin + ( (col-1) * (buttonWidth + W_kButtonGutter) )
				f:ClearAllPoints()
				f:SetPoint("TOPLEFT", "WhammyFrame", "TOPLEFT", x, -y);
				
				petButtonName = "W_Pet_Button"..buttonNum; --for vertical layouts, put the pet button to the right of the unit buttons
				f = getglobal( petButtonName );
				f:ClearAllPoints()
				if ( W_Options.PetButtonLocation == W_kPetButtonRight ) then
					f:SetPoint("TOPLEFT", buttonName, "TOPRIGHT", 0, 0)
				elseif ( W_Options.PetButtonLocation == W_kPetButtonBottom ) then
					f:SetPoint("TOPRIGHT", buttonName, "BOTTOMRIGHT", 0, 0)
				end
				
				buttonNum = buttonNum + 1
			end
			y = y + buttonHeight + W_kButtonGutter;
		end
	end
	
	W_Set_PetButton_Curse_Position() --now that buttons have been moved around, set the pet button curse letters to be in the correct position
end

-- create all of the potential unit buttons. They start off hidden except for #1, which by default is the player
function W_Create_Unit_Buttons()
	--W_ShowMessage("Creating Whammy Buttons")
	local buttonName, b;

	for buttonNum = 1,40 do
		buttonName = "W_Unit_Button"..buttonNum;
		b = CreateFrame("Button", buttonName, WhammyFrame, "WhammyUnitButtonTemplate");
		b:SetID(buttonNum);
		b:Hide(); --start off hidden
	end
	
	W_Set_Button_Size(); --now set the size of all of the buttons to their default size
end

function W_Set_Button_Size()
	--W_ShowMessage("Setting Whammy Button Size")
	local buttonWidth, buttonHeight, buttonName, b;
	
	buttonWidth, buttonHeight = W_Get_Button_Size()
	
	for buttonNum=1,40 do --set the size of all buttons
		buttonName = "W_Unit_Button"..buttonNum;
		b = getglobal( buttonName );
		
		b:SetWidth( buttonWidth ); --set the default width and height for each button
		b:SetHeight( buttonHeight );
		
		getglobal( buttonName.."_HealthBar" ):SetWidth( buttonWidth )
		getglobal( buttonName.."_HealthBar" ):SetHeight( W_kHealthBarHeight )
	
		getglobal( buttonName.."_Curse" ):SetWidth( buttonWidth );
		getglobal( buttonName.."_Curse" ):SetHeight( W_kCurseHeight )
	
		getglobal( buttonName.."_CurseLeft"):SetWidth( W_kCurseLeftSideWidth );
		getglobal( buttonName.."_CurseLeft"):SetHeight( W_kCurseSidesHeight );

		getglobal( buttonName.."_CurseRight"):SetWidth( W_kCurseRightSideWidth );
		getglobal( buttonName.."_CurseRight"):SetHeight( W_kCurseSidesHeight );
		
		W_Adjust_Unit_Name_Width( buttonName );
	end
	
end

function W_Get_Button_Size()
	local buttonWidth, buttonHeight;
	
	if ( W_Options.ButtonSize == W_kButtonSizeSmall ) then --buttons are small size
		buttonWidth = W_kUnitButtonWidthSmall;
		buttonHeight = W_kUnitButtonHeightSmall;
	else --buttons are standard
		buttonWidth = W_kUnitButtonWidth;
		buttonHeight = W_kUnitButtonHeight;
	end

	return buttonWidth, buttonHeight;
end

-- End Unit button creation


-- handle our one and only slash command
function W_Slash_Command_Handler(msg)
	--W_ShowMessage("slash handler");
	local frame = WhammyFrame;
	if ( frame ) then
		if ( frame:IsVisible() ) then
			frame:Hide();
			W_Options.ShowWhammy = false;
			if ( WhammyConfigFrame:IsVisible() ) then
				WhammyConfigFrame:Hide();
			end
		else
			frame:Show();
			W_Options.ShowWhammy = true;
		end
	end
end

-- get an id from the unitID - pass in either party1-party4 or partypet1-partypet4 or raid1-raid40 or raidpet1-raidpet40
function W_GetID( unitID ) --unit type will be either unit or pet
	local id, numberFound;
	
	if ( W_IsUnit( unitID ) ) then -- make sure this is unit, not a pet
		if ( GetNumRaidMembers() == 0 ) then -- party
			id, numberFound = gsub( unitID, "party([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		else
			id, numberFound = gsub( unitID, "raid([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		end
	else -- it was a pet, get it's id
		if ( GetNumRaidMembers() == 0 ) then
			id, numberFound = gsub( unitID, "partypet([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		else
			id, numberFound = gsub( unitID, "raidpet([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		end
	end
	
	if ( numberFound > 0 ) then
		return id;
	else
		return nil;
	end
end

function W_IsUnit( unitID )
	if ( string.find( unitID, "pet") == nil ) then --look for pet in the unitID, which could be pet, partypet or raidpet
		return true
	else
		return false
	end
end

--function W_IsPet( unitID )
--	if ( string.find( unitID, "partypet") ~= nil ) then
--		return true
--	else
--		return false
--	end
--end

function W_IsRaid()
	local numRaidMembers = GetNumRaidMembers();
	
	if ( numRaidMembers == 0 ) then
		return false;
	else
		return true;
	end
end

function W_IsParty()
	local numPartyMembers = GetNumPartyMembers();
	
	if ( not W_IsRaid() and numPartyMembers > 0 ) then --if it is not a raid and there are other party members then its just a party
		return true;
	else -- not just a party, could be player or a raid
		return false;
	end
end

-- for a raid: return the unit button the unit is in, or the unit button the raid p
function W_Get_Raid_Unit_Button_From_UnitID( unitID ) --this is passed raid1-40, or raidpet1-40
	local buttonName;
	
	if ( W_IsUnit( unitID ) ) then
		buttonName = W_gUnitButtonTable[ unitID ]; --this table is filled in whenever there is a raid roster update
	else
		buttonName = W_gPetButtonTable[ unitID ];
	end

	return buttonName
end

--retrieve a saved spell for the passed in drop down
function W_Get_Saved_MenuItem( ddName )
	local menuItem, thing;
	
	menuItem = W_gSavedSpells[ ddName ]; --this is the actual name of a spell, Whisper, Trade, Target, W_kMenuRunMacro or W_kMenuCustomList
	
	if (menuItem == W_kMenuRunMacro) then --it was Run Macro, retrieve the macro from the saved macros table for this dropdown
		thing = W_gSavedMacros[ ddName ]; --return the macro name
	elseif (menuItem == W_kMenuCustomList) then -- the thing was Custom List - either an item, spell or macro
		thing = W_gDropDownCustomList[ ddName ]; --get the table that contains the item name and whether its and item, spell or macro
	else --it was not a macro or custom, so just return the spell name
		thing = nil;
	end
	
	return menuItem, thing --macro name will be nil if there is no menu item
end

-- get the type of char this is, and use the english version of the char type. See what the char type is for spell table loading and to see if it has a pet
function W_GetCharClass( unitID )
	local localizedClass, englishClass = UnitClass( unitID );
	return englishClass
end

-- determine if the passed in unitID is a char that has a pet
function W_CharHasAPet( unitID )
	local charType = W_GetCharClass( unitID ) --get the type of char this is in english

	if ( charType == "HUNTER" or charType == "WARLOCK" ) then
		return true;
	end
end